@@ -120,6 +120,12 @@ class InputData:
             elif label == "ising-model":
                 self.validate_ising_model(content)
                 self.data[label] = content
+            elif label == "training-data":
+                self.validate_training_data(content)
+                self.data[label] = content
+            elif label == "inference-data":
+                self.validate_inference_data(content)
+                self.data[label] = content
             else:
                 self.data[label] = content
         except (OverflowError, TypeError, ValueError):
@@ -275,6 +281,69 @@ class InputData:
                 if not isinstance(interaction["value"], (int, float)):
                     raise Exception("'value' must be a numeric type (int or float).")
 
+    def validate_training_data(self, training_data):
+        vector_size = None
+        if not isinstance(training_data, list):
+            raise Exception("The 'training_data' must be a list of dictionaries.")
+        for data in training_data:
+            if not isinstance(data, dict):
+                raise Exception("The 'training_data' must be a list of dictionaries.")
+            if not "data-point" in data:
+                raise Exception(
+                    "Each dictionary in the list 'training_data' must contain a 'data-point' key."
+                )
+            vector = data["data-point"]
+            if not isinstance(vector, list):
+                raise Exception(
+                    "The 'data-point' value must be a list of numeric values."
+                )
+            if not all(isinstance(item, (int, float)) for item in vector):
+                raise Exception(
+                    "The 'data-point' value must be a list of numeric values (int or float)."
+                )
+            if vector_size is None:
+                vector_size = len(vector)
+            if len(vector) != vector_size:
+                raise Exception(
+                    "All 'data-point' vectors in training data entries must have the same length."
+                )
+            if not "label" in data:
+                raise Exception(
+                    "Each dictionary in the list of training data points must contain a 'label' key."
+                )
+            label = data["label"]
+            if not isinstance(label, (int, float)):
+                raise Exception(
+                    "The 'label' value must be a numeric type (int or float)."
+                )
+
+    def validate_inference_data(self, inference_data):
+        vector_size = None
+        if not isinstance(inference_data, list):
+            raise Exception("The 'inference_data' must be a list of dictionaries.")
+        for data in inference_data:
+            if not isinstance(data, dict):
+                raise Exception("The 'inference_data' must be a list of dictionaries.")
+            if not "data-point" in data:
+                raise Exception(
+                    "Each dictionary in the list of inference data points must contain a 'data-point' key."
+                )
+            vector = data["data-point"]
+            if not isinstance(vector, list):
+                raise Exception(
+                    "The 'data-point' value must be a list of numeric values."
+                )
+            if not all(isinstance(item, (int, float)) for item in vector):
+                raise Exception(
+                    "The 'data-point' value must be a list of numeric values (int or float)."
+                )
+            if vector_size is None:
+                vector_size = len(vector)
+            if len(vector) != vector_size:
+                raise Exception(
+                    "All 'data-point' vectors in inference data entries must have the same length."
+                )
+
     def validate_and_serialize_pub(self, pub):
         shots = None
         paramaters = None
@@ -396,7 +465,8 @@ class InputData:
 
 class StrafulProvider:
 
-    _asp_net_port = "5001"
+    _asp_net_port_dev = "5001"
+    _asp_net_port_prod = "443"
     _key_cloak_port = "8443"
     _client_id = "straful-client"
     _realm_name = "straful-realm"
@@ -410,7 +480,11 @@ class StrafulProvider:
         self._token_expiration_time = None
         self._refresh_token_expiration_time = None
         self._provider_url = url.rstrip("/")
-        self._asp_net_url = f"{self._provider_url}:{self._asp_net_port}"
+        self._asp_net_url = (
+            f"{self._provider_url}:{self._asp_net_port_dev}"
+            if self._debug
+            else f"{self._provider_url}:{self._asp_net_port_prod}"
+        )
         self._auth_call_back_url = f"{self._asp_net_url}/auth/callback"
         self._show_code_callback_url = f"{self._asp_net_url}/auth/showcode"
         self._keycloak_server_url = f"{self._provider_url}:{self._key_cloak_port}"
@@ -791,41 +865,19 @@ In case the service has been recently started please wait 5 minutes for it to be
 
     def _get_autehntication_code(self):
 
-        timeout_seconds = 300
-        start_time = time.time()
-        delta_t = time.time() - start_time
-
-        while delta_t < timeout_seconds:
-
-            if delta_t < 10:
-                time.sleep(1)
-            else:
-                time.sleep(3)
-
-            delta_t = time.time() - start_time
-
-            try:
-                response = requests.get(
-                    self._show_code_callback_url,
-                    params={"state": self._state},
-                    verify=self._use_https,
-                )
-                # TODO: what if I use a wrong email account
-                if response.status_code == 400:
-                    if response.text == "Authorization state is missing.":
-                        raise AuthorizationFailure(
-                            "Authorization state not found on remote server."
-                        )
-                    continue
-                data = response.text
-                auth_code = data.split(": ")[1]
-                return auth_code
-            except requests.RequestException as e:
-                raise AuthorizationFailure(
-                    f"Remote server is not responding to attempts to retrieve authorization code, exception is {e}."
-                )
-
-        raise AuthorizationFailure("Authorization code not received.")
+        response = requests.get(
+            self._show_code_callback_url,
+            params={"state": self._state},
+            verify=self._use_https,
+        )
+        # TODO: what if I use a wrong email account
+        if response.status_code == 400:
+            raise AuthorizationFailure(
+                "Authorization code was not received. Please make sure you are using a Google account which you have signed-on our web-site. If our website is not online and please try again later."
+            )
+        data = response.text
+        auth_code = data.split(": ")[1]
+        return auth_code
 
     def is_valid_uuid(self, value: str) -> bool:
         try:
